//
//  AASFileItem.h
//
//  Created by Andreas Schwarz on Thu Apr 18 2002.
//  OS X 10.5 or later required.
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/* Please note that this object is only KVO compliant for certain properties as described below. */


#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/mount.h>


#pragma mark Keys

/* Keys for archive dictionaries */
extern NSString *AASArchiveDataKey;		// Will return an NSData object containing an AliasHandle
extern NSString *AASArchivePathKey;		// Will return the last known URL of the file




#pragma mark -
#pragma mark Structures

struct AASFileInfoList {
	id owner;
	struct timespec modTimespec;
	struct timespec changeTimespec;
	UInt8 *pathRepresentation;
	struct AASFileInfoList *nextItem;
	NSURL *pathURL;
	HFSUniStr255 fileName;
};

struct AASFileChanges {
	BOOL fileValid;
	BOOL pathChanged;
	BOOL nameChanged;
	BOOL fileModified;
	BOOL newChanges;
	BOOL iNodeChanged;
};



#pragma mark -



@interface AASFileItem : NSObject <NSCopying> {
	// Declared protected so that subclasses in this framework can have fast access to variables in specific instances.
	// DO NOT attempt to access these variables yourself
	@protected
    NSString *_path;
    NSURL *_pathURL;
    FSRef _fsRef;
	NSURL *_lastValidURL;

	struct AASFileInfoList *_pathInfo;
	
    unsigned int _deviceNumber;
    unsigned int _iNode;
    unsigned int _hashValue;
	
    NSString *fileName;
	
    BOOL refValid;
    NSDate *modificationDate;
	
    BOOL isEjectable;
    BOOL isAVolumeRoot;
    BOOL isVolumesDirectory;
    BOOL isRootDirectory;
    BOOL isBasicDirectory;
    BOOL isSymlink;
	BOOL isLocal;
	BOOL doNotBrowse;
	BOOL onRootFilesystem;
	BOOL fileSupportsLabels;
	
	BOOL isPlaceholder;
	
	@private
    NSMapTable *_container;
	
	AASFileItem *parentItem;
	
	NSDictionary *latestEvent;
	
	NSInteger lastCheckID;
	struct AASFileChanges lastChanges;
	struct AASFileInfoList *lastInfoListItem;
}


/* Returns the number of AASFileItem instances currently in existence. */
+ (NSUInteger)numberOfFilesTracked;



#pragma mark -
#pragma mark Creating File Items


/* Designated initializer. Do not use this to create a new instance (use newFileItemForURL: or equivalents, which will create a new instance and call the initializer only if necessary). Subclasses will want to implement this. ref, pathRep, uniName, and fileInfo may be NULL. */
- (id)initWithURL:(NSURL *)url path:(NSString *)p fsRef:(FSRef *)ref pathRep:(UInt8 *)pathRep statInfo:(struct stat *)fileInfo catInfo:(FSCatalogInfoPtr)catInfo unicodeName:(HFSUniStr255 *)uniName;

- (id)copyWithZone:(NSZone *)zone;



#pragma mark -
#pragma mark Getting File Items


/* Returns an AASFileItem object for the specified URL. If an instance already exists for that url, that will be reused (this is nice when you are using subclasses of AASFileItem that may contain a lot of information about a file; saves memory and time).  The returned value is not autoreleased; this works just like [[AASFileItem alloc] init], except for reusing instances when possible. */
+ (id)newFileItemForURL:(NSURL *)url;

/* Equivalent to -newFileItemForURL: but will make use of the FSRef if it becomes necessary to create a new instance (this is more efficient than -newFileItemForURL: or -newFileItemForFSRef: if you already have both handy). */
+ (id)newFileItemForURL:(NSURL *)url fsRef:(FSRef *)ref;

/* Equivalent to -newFileItemForURL: but will make use of the path, FSRef, unicode name, and catalog info if it becomes necessary to create a new instance. If not NULL, catInfo must include kFSCatInfoNodeFlags and kFSCatInfoFinderInfo. */
+ (id)newFileItemForURL:(NSURL *)url path:(NSString *)path fsRef:(FSRef *)ref catalogInfo:(FSCatalogInfoPtr)catInfo unicodeName:(HFSUniStr255 *)uniName;

/* Equivalent to -newFileItemForURL: but can take a path. */
+ (id)newFileItemForPath:(NSString *)path;

/* Equivalent to -newFileItemForURL: but can take an FSRef pointer. Returns nil if the FSRef cannot be resolved. */
+ (id)newFileItemForFSRef:(FSRef *)ref;

/* Returns a the file associated with the archive (see -archiveDictionary) and (in the case of subclasses) may add persistent information stored in the archive to the file. Returns nil if file cannot be found. Subclasses should implement this to set persistent information as necessary (see -loadDataIntoArchive: for more). */
+ (id)newFileItemForArchiveDictionary:(NSDictionary *)archive allowPlaceholders:(BOOL)placeholders;
+ (id)newFileItemForArchiveDictionary:(NSDictionary *)archive;



/* Returns whether this file item is a placeholder for a nonexistent file. */
@property(readonly) BOOL isPlaceholder;



#pragma mark -
#pragma mark Converting to/from Arrays

/* Convenience class methods to convert to and from arrays of NSString paths / NSURLs / AliasHandle NSData objects to arrays of AASFileItem instances. If placeholders are not allowed, an exception is made for the volumesFileItemPlaceholder object, because that object always "exists" but must always be a placeholder since it does not represent an actual folder. */
+ (NSArray *)fileItemsForPaths:(NSArray *)paths;
+ (NSArray *)fileItemsForPaths:(NSArray *)paths allowPlaceholders:(BOOL)placeholders;
+ (NSArray *)pathsForFileItems:(NSArray *)files;
+ (NSArray *)fileItemsForURLs:(NSArray *)paths;
+ (NSArray *)fileItemsForURLs:(NSArray *)paths allowPlaceholders:(BOOL)placeholders;
+ (NSArray *)urlsForFileItems:(NSArray *)files;
+ (NSArray *)fileItemsForArchiveDictionaries:(NSArray *)data;
+ (NSArray *)fileItemsForArchiveDictionaries:(NSArray *)data allowPlaceholders:(BOOL)placeholders;
+ (NSArray *)archiveDictionariesForFileItems:(NSArray *)files;


 


#pragma mark -
#pragma mark Hashing

/* So they can be used as a key in an NSDictionary, file objects have a unique hash and respond to -isEqual: based on the associated file's device number and inode.  KVO compliant (but value should never change). */
@property(readonly) unsigned int hash;
@property(readonly) NSString * hashString;
@property(readonly) unsigned int deviceNumber;
@property(readonly) unsigned int iNode;
@property(readonly) void * iNodeHash;




#pragma mark -
#pragma mark File Information




/* KVO Compliant file info properties. */

/* Returns whether this file item object is valid.  It is no longer valid if it points to a file that has been deleted or otherwise lost track of. In this case, -path, -url, -currentURL and related methods will also return nil or NULL. KVO compliant. */
@property(readonly) BOOL isValid;

/* Returns the last known path to the file, without checking if it has changed.  This is the most efficient method to get the path. Returns nil if the file cannot be found. KVO compliant. */
@property(readonly, copy) NSString * path;

/* Returns the last known url to the file, without checking if it has changed.  This is the most efficient method to get the url. Returns nil if the file cannot be found.  KVO compliant. */
@property(readonly, copy) NSURL * url;

/* Returns the last known modification date for the file. KVO compliant. */
@property(readonly, copy) NSDate * modificationDate;


/* Returns the true file name of the file (including the extension).  This is really only the last path component, so it may not work properly on things like volume names. KVO compliant. */
@property(readonly) NSString * fileName;

/* Returns fileName. Subclasses can override this to display something more appropriate if desired. KVO compliant. */
@property(readonly) NSString * displayName;

/* Returns whether this file is a directory.  This includes file packages and bundles, but does not include aliases or symlinks to directories. KVO compliant (but value should never change). */
@property(readonly) BOOL isBasicDirectory;

/* Returns whether this file is a symbolic link. KVO compliant (but value should never change). */
@property(readonly) BOOL isSymlink;

/* Returns whether this file is the volumesFileItemPlaceholder object (the placeholder object that represents the "Computer" level of the filesystem, which shows all mounted volumes).  KVO compliant (but value should never change). */
@property(readonly) BOOL isVolumesDirectory;

/* Returns whether this file is the root level of some mounted volume. KVO compliant (but value should never change). */
@property(readonly) BOOL isAVolumeRoot;

/* Returns whether this file is the root level of a volume that can be ejected. KVO compliant (but value should never change). */
@property(readonly) BOOL isEjectable;

/* Returns whether this file is the root directory of the filesystem, "/". KVO compliant (but value should never change). */
@property(readonly) BOOL isRootDirectory;

/* Returns whether this file is on a local filesystem. KVO compliant (but value should never change). */
@property(readonly) BOOL isLocal;

/* Returns whether this file is on the root filesystem. KVO compliant (but value should never change). */
@property(readonly) BOOL onRootFilesystem;


/* Returns whether this file is on a filesystem that shouldn't ever be visible to the user (the mounts /Net and /Home and /dev are a few examples). KVO compliant (but value should never change). */
@property(readonly) BOOL doNotBrowse;


/* Returns the file item for the parent folder of this file item. KVO compliant. */
@property(readonly) id parentItem;





/* File info properties that are not KVO compliant. */

/* Returns a copy of the last known path in the filesystem representation, without checking if it has changed.  This is the most efficient method to get the filesystem representation.  Returns NULL if the file cannot be found. If not NULL, the return value must be freed with free() when the caller is done with it. */
@property(readonly) UInt8 * filesystemRepresentation;


/* Returns a pointer to an FSRef for the file (might be NULL if an FSRef could not be created or if the file cannot be found). */
@property(readonly) FSRef * fsRef;

/* Returns a dictionary containing data that can be saved to disk and will allow the file item and any stored information to be recreated later. */
@property(readonly) NSDictionary * archiveDictionary;


/* Returns the current path of the file even if it has moved.  This is much, much more expensive than calling -(NSString *)path and really shouldn't be necessary unless you're not using an AASDirectorySubscription to track your files (usually through the AASFileManager instance). Returns nil if the file cannot be found. */
@property(readonly) NSString * currentPath;

/* Returns a copy of the current path in the filesystem representation of the file even if it has moved.  This is much, much more expensive than calling -(NSString *)path and really shouldn't be necessary unless you're not using an AASDirectorySubscription to track your files (usually through the AASFileManager instance).  Returns NULL if the file cannot be found. If not NULL, the return value must be freed with free() when the caller is done with it. */
@property(readonly) UInt8 * currentFilesystemRepresentation;

/* Returns the current URL of the file even if it has moved.  This is much, much more expensive than calling -url and really shouldn't be necessary unless you're not using an AASDirectorySubscription to track your files (usually through the AASFileManager instance). Returns nil if the file cannot be found. */
@property(readonly) NSURL * currentURL;

/* Returns the current modification date for the file even if it has changed since the file object was last updated. More expensive than just calling -modificationDate.  Returns nil if the file cannot be found. */
@property(readonly) NSDate * currentModificationDate;


/* If the file cannot be found (isValid = NO), this method will return the last known URL. */
@property(readonly) NSURL * lastValidURL;

/* Returns whether this file is on a filesystem that supports label attributes and is writable. */
@property(readonly) BOOL fileSupportsLabels;




/* Permissions. Always returns the current state of the file (this information is not cached) */
@property(readonly) BOOL isReadable; // Traverses symlinks
@property(readonly) BOOL isWritable; // Traverses symlinks
@property(readonly) BOOL isDeletable;
@property(readonly) BOOL isExecutable; // Traverses symlinks
@property(readonly) BOOL isRenamable;
@property(readonly) BOOL isSeeable; // Parent folder is readable






#pragma mark -
#pragma mark Observers and Events

/* This is used by the -updateForObserver: method as well as by AASDirectorySubscription objects, but shouldn't be used by anyone else. */
- (struct AASFileChanges *)changesForObserver:(id)observer;

- (BOOL)removeFileObserver:(id)observer;

- (void)registerFileObserver:(id)observer;

/* Called by the AASDirectorySubscription when the file has been changed in some way (the event dictionary is the same as what gets passed to subscribers to the AASDirectorySubscription).  Checks to see if the file has already updated for this event during this check cycle, and if not, calls updateForEvent: on itself.  See the updateForEvent: method's comments for more. */
- (void)eventOccurred:(NSDictionary *)event;

/* This is useful in case you are using AASFileItem objects alone, without the AASDirectorySubscription.  When you call this, it will check to see if the file has changed/moved since the last time this method was called by that observer.  If it has changed, it will update itself through the updateForEvent: method and then return the update dictionary.  Returns nil if no changes have occurred.  Updates by this method won't confuse any AASFilesystemSubscriptions that may be using this file object, they can be used together safely. */
- (NSDictionary *)updateForObserver:(id)observer;


/* Returns the most recent event passed to the eventOccurred: method. KVO compliant. */
@property(readonly, copy) NSDictionary *latestEvent;


#pragma mark -
#pragma mark Subclassable Methods


/* The updateForEvent: method is there in case subclassers want to implement some kind of automatic file-information updates whenever the file is known to change.  It is called on the object before any subscribers are sent the notification about the change in the file, so the information that is updated will be ready if the subscribers expect it.  For example, if the subclass includes information on the file's file size, when the file recieves an AASFileUpdatedEvent the subclass can check/update the file size value, so that later when subscribers get notified of the update, they can immediately access the newly updated file size information.  Also, if you are using the subclass in places without a AASDirectorySubscription (perhaps relying on periodic calls to the update method), you can override this in your subclass to notify any interested objects that the subclass instance has changed.  */
- (void)updateForEvent:(NSDictionary *)event;


/* Called by the -archiveDictionary method to allow subclasses to add persistent information to the archive dictionary. Subclasses must call 'super.' Subclasses can make use of this information when overriding +newFileItemForArchiveDictionary: (For example, they can call 'super' and then set any necessary attributes to the returned file object before passing it along). Be aware the object you recieve in newFileItemForArchiveDictionary: may already have been in use, so you may wish to check that your values haven't been superceded by new information before setting them.  */
- (void)loadDataIntoArchive:(NSMutableDictionary *)dictionary;


/* Called when a file item changes placeholder status. At the point where this is called, isPlaceholder returns the current (new) value. Subclasses should use this to update whatever custom file information might need updating. Call the super implmentation FIRST to ensure all properties of AASFileItem have already been updated before you rely on their values. In addition, after this method returns, the object will receive an updateForEvent: message with an AASFileAppearedEvent - it is possible that method may already update all or most of the information in your subclass, so you may find you do not need to use this method after all, or may want to update only a particular subset of variables to keep from duplicating your efforts. */
- (void)placeholderStatusChanged;

@end
