//
//  AASFileLoadOperation.h
//  AASFilesystemTools
//
//  Created by Andreas Schwarz on 3/6/08.
//  OS X 10.5 or later required.
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* This NSOperation subclass reads directory contents from disk, creates file objects for each file, and adds the file to an array based on optional file filter criteria. Directory contents are loaded for every directory added to the input queue. If recursively loading subdirectories, a directory filter can be used to determine what should be considred a valid directory, and where that directory's contents are. Recursively loaded contents are all added individually to the results array and no order is guaranteed (eg. they will not be grouped by directory). Recursive operations only iterate through a directory once; if the directory filter returns the same directory twice (typically, when you're iterating through directories that contain aliases or symbolic links to directories that show up elsewhere down the same tree and your filter instructs the operation to follow those links) it will be ignored. Files in the loading array are guaranteed to be unique (no duplicates) for recursive loads only if the input queue has not been modified while the operation has run. */


#import <Cocoa/Cocoa.h>
#import "AASQueuedObjectOperation.h"

@class AASFileItem;
@class AASFileFilter;
@class AASDirectoryFilter;


@interface AASFileLoadOperation : AASQueuedObjectOperation {	
	AASFileFilter *fileFilter;
	AASDirectoryFilter *directoryFilter;
	
	CFIndex depth;
	BOOL includeTopLevel;
	
	NSDictionary *excludePaths;
}



/* Get the file filter that will tell the operation whether or not to include a file in the loading queue. Do not change targets or selectors after the operation has been added to an operation queue. Defaults to AASFileFilter class defaults. */
@property(retain) AASFileFilter *fileFilter;

/* Get the AASDirectoryFilter that this operation will use to determine what is a directory and where its contents are. Uses AASDirectoryFilter defaults by default, but you will probably want to set targets and selectors to filter out things like packages, and return the real directory for things like aliases to directories. Do not change after the operation has been added to an operation queue. */
@property(retain) AASDirectoryFilter *directoryFilter;

/* Set or get the how many levels deep the recursive operation should go. '-1' is unlimited. '0' essentially will not do more than filter the files already given. Defaults to '1'. Do not set after the AASFileLoadOperation has been added to an operation queue. */
@property CFIndex depth;

/* Default is NO. If YES, the files in the queue may be included in the output. If NO, only the files loaded recursively for the original directories in the queue will be returned. */
@property BOOL includeTopLevel;

/* Set a list of paths to files that you wish to exclude from the results. Keys to this dictionary should be the NSString paths; values can be anything. */
@property(retain) NSDictionary *excludePaths;


@end
